﻿Imports RMCLinkNET

Public Class Form1

    Dim xPosLim, xNegLim, yPosLim, yNegLim
    Dim lastPosX() As Single = New Single(100) {}
    Dim lastPosY() As Single = New Single(100) {}
    Dim posIndex As Integer
    Dim trailLen As Integer
    Dim startupTrailCntr As Integer = 0
    Dim dotWidth As Single = 6
    Dim DataItems As Integer = 5
    Dim LastPlotID As Integer = 0

    Private RMC As RMCLink

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Since we are disconnected to start, enable and disable buttons and labels accordingly.
        connectButton.Enabled = True
        disconnectButton.Enabled = False
        'initialize default plot items
        If My.Settings.PlotDataItems Is Nothing Then
            My.Settings.PlotDataItems = New Specialized.StringCollection
            My.Settings.PlotDataItems.Add("Targets Pos")
            My.Settings.PlotDataItems.Add("pu")
            My.Settings.PlotDataItems.Add("Actual Pos")
            My.Settings.PlotDataItems.Add("pu")
            My.Settings.PlotDataItems.Add("Target Vel")
            My.Settings.PlotDataItems.Add("pu/s")
            My.Settings.PlotDataItems.Add("Actual Vel")
            My.Settings.PlotDataItems.Add("pu/s")
            My.Settings.PlotDataItems.Add("Ctrl Output")
            My.Settings.PlotDataItems.Add("%")
        End If
        Dim dt As DataTable = New Data.DataTable("ParentTable")
        dt.Columns.Add("Item Name", GetType(String))
        dt.Columns.Add("Units", GetType(String))
        Me.dgPlotItems.DataSource = dt
        'restore saved plot items
        Dim i As Integer
        For i = 0 To My.Settings.PlotDataItems.Count - 1
            dt.Rows.Add(New String() {My.Settings.PlotDataItems.Item(i), My.Settings.PlotDataItems.Item(i + 1)})
            i += 1
        Next

    End Sub

    Private Sub Form1_Closing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        If RMC IsNot Nothing Then
            If RMC.IsConnected(PingType.DoNotPing) Then
                RMC.Disconnect()
            End If
        End If
        Dim dt As DataTable = New Data.DataTable("PlotItems")
        dt = CType(Me.dgPlotItems.DataSource, DataTable)
        Dim strColl As New System.Collections.Specialized.StringCollection()
        For Each row As DataRow In dt.Rows
            strColl.Add(row(0).ToString)
            strColl.Add(row(1).ToString)
        Next
        My.Settings.PlotDataItems = strColl
        My.Settings.Save()
    End Sub

    Private Sub connectButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles connectButton.Click
        Try
            Connect()
        Catch ex As ConnectionNotMadeException
            MessageBox.Show("Unable to connect. " + ex.Message)
        End Try
    End Sub

    Private Sub disconnectButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles disconnectButton.Click
        RMC.Disconnect()
        Disconnect()
    End Sub
    Private Sub Connect()
        ' Create an RMCLink object.
        ' "rmc70.deltamotion.com" is the DNS name for an RMC70 provided by Delta 
        ' that is always available through the web. 
        ' Typically, an IP address such as "192.168.0.161" would be used.
        Select Case rmcType.Text
            Case "RMC70"
                RMC = RMCLink.CreateEthernetLink(DeviceType.RMC70, IPAddrValue.Text)
            Case "RMC150"
                RMC = RMCLink.CreateEthernetLink(DeviceType.RMC150, IPAddrValue.Text)
            Case "RMC200"
                RMC = RMCLink.CreateEthernetLink(DeviceType.RMC200, IPAddrValue.Text)
        End Select

        RMC.Connect()

        ' We have successfully connected, so update which buttons are enabled.
        disconnectButton.Enabled = True
        connectButton.Enabled = False
        rmcType.Enabled = False
        IPAddrValue.Enabled = False
        btnAutoPlotUpload.Enabled = True
        tbConnectedStatus.Text = "CONNECTED"
        tbConnectedStatus.ForeColor = Color.Green
        ConnectionTimer.Enabled = True
        ButtonSingleUpload.Enabled = True

    End Sub

    Private Sub Disconnect()

        ' We have successfully disconnected, so update which buttons are enabled.
        connectButton.Enabled = True
        disconnectButton.Enabled = False
        startupTrailCntr = 0
        rmcType.Enabled = True
        IPAddrValue.Enabled = True

        'Clear display
        Dim Gr As Drawing.Graphics
        Gr = PlotDisplay.CreateGraphics
        Gr.Clear(Color.White)
        tbConnectedStatus.Text = "DISCONNECTED"
        tbConnectedStatus.ForeColor = Color.Red
        rdoPlotScan.Checked = False
        btnAutoPlotUpload.Enabled = False
        btnStopAutoMode.Enabled = False
        ButtonSingleUpload.Enabled = False

        ' Stop Timer
        Timer1.Stop()

    End Sub
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        CheckForPlot()
        If rdoPlotScan.Checked Then
            rdoPlotScan.Checked = False
        Else
            rdoPlotScan.Checked = True
        End If
    End Sub
    Private Sub CheckForPlot()
        Dim plotState As Integer
        Dim plotID As Integer
        Dim plotConfig As Integer
        Dim dataItems, plotSamples, plotPeriod, regAddr
        plotSamples = 200
        dataItems = 5
        plotPeriod = 2
        regAddr = 100
        If Me.WindowState <> FormWindowState.Minimized Then
            Try
                If RMC.IsConnected(PingType.Ping) Then
                    plotConfig = Val(ComboBoxPlotConfig.Text)
                    GetPlotIDandState(plotConfig, plotID, plotState)
                    If (plotState = 2) And Not (plotID = LastPlotID) Then
                        ReadAndDisplayPlot(plotID)
                        tbPlotStatus.Text = "LOADED"
                    ElseIf plotState = 1 Then
                        tbPlotStatus.Text = "Loading.."
                    ElseIf plotState = 0 Or plotID = LastPlotID Then
                        tbPlotStatus.Text = "Waiting for new plot"
                    End If
                End If
            Catch ex As ReadWriteFailedException
                MessageBox.Show("Unable to read data. " + ex.Message)
                Disconnect()
            End Try
        End If
    End Sub
    Private Sub btnAutoPlotUpload_Click(sender As Object, e As EventArgs) Handles btnAutoPlotUpload.Click
        btnAutoPlotUpload.Enabled = False
        btnStopAutoMode.Enabled = True
        ButtonSingleUpload.Enabled = False
        Timer1.Start()
    End Sub


    Private Sub btnStopAutoMode_Click(sender As Object, e As EventArgs) Handles btnStopAutoMode.Click
        StopAutoMode()
    End Sub
    Private Sub StopAutoMode()
        btnStopAutoMode.Enabled = False
        btnAutoPlotUpload.Enabled = True
        rdoPlotScan.Checked = False
        ButtonSingleUpload.Enabled = True
        Timer1.Stop()
    End Sub
    Private Sub Form1_Resize(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        Dim horzSpace, vertSpace, vertPU, horzPU As Double
        Dim ratioSpace, ratioPU As Double                    'ratio is y over x

        horzSpace = Me.ClientSize.Width - 600
        vertSpace = Me.ClientSize.Height - 40

        If xPosLim <> Nothing Then
            ratioSpace = vertSpace / horzSpace
            horzPU = xPosLim - xNegLim
            vertPU = yPosLim - yNegLim
            ratioPU = vertPU / horzPU

            If ratioPU <= ratioSpace Then
                PlotDisplay.Width = horzSpace
                PlotDisplay.Height = ratioPU * PlotDisplay.Width
            Else
                PlotDisplay.Height = vertSpace
                PlotDisplay.Width = PlotDisplay.Height / ratioPU
            End If
        Else
            PlotDisplay.Width = horzSpace
            PlotDisplay.Height = vertSpace
        End If

    End Sub
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    'GetPlotConfigData
    'Gets info regarding the plot that will be needed for reading the plot, etc.
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Private Sub GetPlotConfig(plotConfig As Integer, ByRef dataItems As Integer, ByRef plotSamples As Integer,
                          ByRef capturedSamples As Integer, ByRef plotPeriod As Single, ByRef regAddr() As String)


        Dim i
        Dim PlotData() As Single = New Single(32) {}
        Dim inPlotData() As Integer = New Integer(32) {}

        Select Case rmcType.Text
            Case "RMC70"         'RMC70
                RMC.ReadFFile(32 + plotConfig, 1, PlotData, 0, 32)
                plotSamples = PlotData(0)
                plotPeriod = PlotData(1)
            Case "RMC150"     'RMC150
                RMC.ReadFFile(96 + plotConfig, 1, PlotData, 0, 32)
                plotSamples = PlotData(0)
                plotPeriod = PlotData(1)
            Case "RMC200"      'RMC200
                RMC.ReadLFile(512 + plotConfig, 1, inPlotData, 0, 32)
                RMC.ReadFFile(512 + plotConfig, 2, PlotData, 0, 1)
                plotPeriod = PlotData(0)
                plotSamples = inPlotData(0)

                For i = 0 To 31
                    PlotData(i) = CSng(inPlotData(i))
                Next i
        End Select

        capturedSamples = PlotData(9)

        'Extract the number of data items and register addresses from the plot config
        dataItems = 0
        i = 0
        While i < 16 And PlotData(15 + i) <> 0
            dataItems = dataItems + 1
            regAddr(i) = "%MD" + CStr(Int(PlotData(15 + i) / 4096)) + "." +
                             CStr(PlotData(15 + i) Mod 4096)
            i = i + 1
        End While
        dataItems = i
    End Sub

    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    'GetPlotIDandState
    'Gets the plot ID and State
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Private Sub GetPlotIDandState(plotConfig As Integer, ByRef plotID As Integer, ByRef plotState As Integer)

        Dim PlotData() As Single = New Single(1) {}
        Dim inPlotData() As Integer = New Integer(1) {}

        Select Case rmcType.Text
            Case "RMC70"         'RMC75
                RMC.ReadFFile(32 + plotConfig, 8, PlotData, 0, 2)
                plotID = PlotData(0)
                plotState = PlotData(1)  'Plot State  0 = Not triggered, 1 = capturing, 2 = complete
            Case "RMC150"     'RMC150
                RMC.ReadFFile(96 + plotConfig, 8, PlotData, 0, 2)
                plotID = PlotData(0)
                plotState = PlotData(1)  'Plot State  0 = Not triggered, 1 = capturing, 2 = complete
            Case "RMC200"      'RMC200
                RMC.ReadLFile(512 + plotConfig, 8, inPlotData, 0, 2)
                plotID = inPlotData(0)
                plotState = inPlotData(1)  'Plot State  0 = Not triggered, 1 = capturing, 2 = complete        
        End Select




    End Sub

    Private Sub ConnectionTimer_Tick(sender As Object, e As EventArgs) Handles ConnectionTimer.Tick
        Try
            If Not RMC.IsConnected(PingType.Ping) Then
                Disconnect()
            End If
        Catch ex As Exception
            Disconnect()
        End Try
    End Sub

    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    'ReadPlot
    'This reads the requested plot configuration using the dynamic upload area,
    'which allows reading very long plots. The RMCTools help describes how to use the dynamic
    ' upload area in the topic "Reading RMC Plots with a Host Controller".
    '
    'plotConfig =  the number of the plot to read. From RMCTools project pane > Plot Configurations
    'dataItems = number of data sets in the plot
    'plotSamples = number of plot samples
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Private Function ReadPlot(plotConfig, dataItems, plotSamples, capturedSamples) As Single(,)

        Dim plotSet(0, 0) As Single                                 '(data item, sample)
        Dim addrFilePlots, readLength As Integer
        Dim PlotDataArray() As Single = New Single(1) {}
        Dim WriteData() As Integer = New Integer(1) {}
        Dim plotInfo() As Integer = New Integer(4) {}
        Dim i, j, k As Integer

        Select Case rmcType.Text
            Case "RMC70"      'RMC70
                addrFilePlots = 40
            Case "RMC150"     'RMC150
                addrFilePlots = 104
            Case "RMC200"      'RMC200
                addrFilePlots = 576
            Case Else
                addrFilePlots = 0
        End Select

        WriteData(0) = 1                            'Write 1 to the Upload Mode/Status Register and write the number of requested read samples
        WriteData(1) = 200                          'number of samples read each time = (200 x plotItems)
        RMC.WriteLFile(addrFilePlots + plotConfig, 0, WriteData, 0, 2)

        'Read the plot data 200 samples at a time.
        'Each read length will need to be 200 x plotItems + 5, but the 5 will be done with a ReadLFile.
        readLength = 200 * dataItems

        'After each read, extract the data and put in plotSet array
        ReDim plotSet(capturedSamples - 1, dataItems - 1)          '(data item, sample)
        ReDim PlotDataArray(readLength)

        For i = 0 To Int(capturedSamples / 200) - 1                 'int is for truncating

            RMC.ReadLFile(addrFilePlots + plotConfig, 0, plotInfo, 0, 5)
            RMC.ReadFFile(addrFilePlots + plotConfig, 5, PlotDataArray, 0, readLength)

            'Extract the plotSet data
            For j = 0 To dataItems - 1
                For k = 0 To 199

                    plotSet(i * 200 + k, j) = PlotDataArray(j * 200 + k)

                Next k
            Next j
        Next i

        'Do the last read
        If (capturedSamples Mod 200) <> 0 Then
            'Write the number of requested read samples for the last read
            WriteData(0) = capturedSamples Mod 200
            RMC.WriteLFile(addrFilePlots + plotConfig, 1, WriteData, 0, 1)

            'Read the data from the RMC
            RMC.ReadLFile(addrFilePlots + plotConfig, 0, plotInfo, 0, 5)
            RMC.ReadFFile(addrFilePlots + plotConfig, 5, PlotDataArray, 0, (capturedSamples Mod 200) * dataItems)

            'Extract the plotSet data
            For j = 0 To dataItems - 1
                For k = 0 To (capturedSamples Mod 200) - 1
                    plotSet(i * 200 + k, j) = PlotDataArray(j * (capturedSamples Mod 200) + k)
                Next
            Next
        End If

        ReadPlot = plotSet
    End Function

    Private Sub ButtonSingleUpload_Click(sender As Object, e As EventArgs) Handles ButtonSingleUpload.Click
        CheckForPlot()
    End Sub

    'Reads the requested plot.
    'First reads the plot configuration to find out how many data items
    'and samples are in the requested plot and the sample interval.
    'then it reads the plot and places the data items in columns in Excel.
    Private Sub ReadAndDisplayPlot(PlotID As Integer)

        Dim plotSamples, capturedSamples, plotItems, plotConfig As Integer
        Dim plotPeriod As Single
        Dim i, j As Integer
        Dim regAddr(15) As String
        '    Dim plotTime()
        Dim plotTime(0) As Single
        Dim itemNames(0) As String
        Dim Units(0) As String

        plotConfig = CInt(ComboBoxPlotConfig.Text)

        'get the plotSamples, pltoPeriod, plotItems and register addresses
        GetPlotConfig(plotConfig, plotItems, plotSamples, capturedSamples, plotPeriod, regAddr)

        If Not plotItems = (dgPlotItems.Rows.Count - 1) Then
            StopAutoMode()
            MsgBox("error: The plot items list must have the same number items as the plot in the RMC.  RMC Plot items = " + Str(plotItems) + " Display Plot Items = " + Str(dgPlotItems.Rows.Count - 1))
            Exit Sub
        End If

        LastPlotID = PlotID

        For i = 0 To 4

        Next

        'Read the plot data
        Dim plotData(,) = ReadPlot(plotConfig, plotItems, plotSamples, capturedSamples)

        'Make the x axis data (in seconds)
        ReDim plotTime(plotSamples - 1)
        For i = 0 To plotSamples - 1
            plotTime(i) = plotPeriod * i
        Next i

        'add plot items to chart
        ReDim itemNames(plotItems - 1)
        ReDim Units(plotItems - 1)

        PlotDisplay.Series.Clear()
        For i = 0 To plotItems - 1
            itemNames(i) = dgPlotItems.Rows(i).Cells(0).Value
            Units(i) = dgPlotItems.Rows(i).Cells(1).Value
            PlotDisplay.Series.Add(itemNames(i) + " (" + Units(i) + ")")
            PlotDisplay.Series(i).Points.Clear()
            PlotDisplay.Series(i).ChartType = DataVisualization.Charting.SeriesChartType.Line
        Next

        j = 0
        i = 0
        For j = 0 To plotItems - 1
            'For i = 0 To plotSamples - 1
            For i = 0 To UBound(plotData, 1)
                PlotDisplay.Series(j).Points.AddXY(plotTime(i), plotData(i, j))
            Next i
        Next j
        PlotDisplay.Update()        'display chart

        If cbSaveToFile.Checked Then
            If ComboBoxFileDataType.SelectedItem.ToString = "RMC Plot" Then
                PlotCreateXMLFile(plotData, regAddr, itemNames, Units, plotPeriod, PlotID)
            Else
                PlotCreateCSVfile(plotData, regAddr, itemNames, Units, plotPeriod, PlotID)
            End If
        End If

    End Sub

    Private Sub ButtonBrowse_Click(sender As Object, e As EventArgs) Handles ButtonBrowse.Click
        Dim fd As OpenFileDialog = New OpenFileDialog()

        fd.Title = "Open File Dialog"
        fd.InitialDirectory = "C:\"
        fd.Filter = "All files (*.*)|*.*|ini files (*.ini)|*.ini"
        fd.FilterIndex = 2
        fd.RestoreDirectory = True


        If fd.ShowDialog() = DialogResult.OK Then
            TextBoxSettingsFile.Text = fd.FileName
        End If
    End Sub

    Private Sub ButtonSaveToFile_Click(sender As Object, e As EventArgs) Handles ButtonSaveToFile.Click
        Using sw As New System.IO.StreamWriter(TextBoxSettingsFile.Text, False)
            sw.WriteLine(rmcType.Text)
            sw.WriteLine(IPAddrValue.Text)
            sw.WriteLine(ComboBoxFileDataType.Text)
            sw.WriteLine(txtPlotPath.Text)
            sw.WriteLine(txtPlotName.Text)
            sw.WriteLine(ComboBoxPlotConfig.Text)
            sw.WriteLine(cbSaveToFile.Checked)
            Dim dt As DataTable = New Data.DataTable("PlotItems")
            dt = CType(Me.dgPlotItems.DataSource, DataTable)
            For Each row As DataRow In dt.Rows
                sw.WriteLine(row(0).ToString)
                sw.WriteLine(row(1).ToString)
            Next
        End Using

    End Sub

    Private Sub ButtonReadFromFile_Click(sender As Object, e As EventArgs) Handles ButtonReadFromFile.Click
        Using sr As New System.IO.StreamReader(TextBoxSettingsFile.Text, False)
            rmcType.Text = sr.ReadLine
            IPAddrValue.Text = sr.ReadLine
            ComboBoxFileDataType.Text = sr.ReadLine
            txtPlotPath.Text = sr.ReadLine
            txtPlotName.Text = sr.ReadLine
            ComboBoxPlotConfig.Text = sr.ReadLine
            cbSaveToFile.Checked = sr.ReadLine
            Dim Col0 As String = ""
            Dim Col1 As String = ""
            Dim dt As DataTable = New Data.DataTable("ParentTable")
            dt.Columns.Add("Item Name", GetType(String))
            dt.Columns.Add("Units", GetType(String))
            Me.dgPlotItems.DataSource = dt
            Col0 = sr.ReadLine
            Col1 = sr.ReadLine
            Do While Col0 IsNot Nothing
                dt.Rows.Add(New String() {Col0, Col1})
                Col0 = sr.ReadLine
                Col1 = sr.ReadLine
            Loop
        End Using

    End Sub

    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    'GetLoopTime
    'Gets the loop time of the controller.
    '
    'rmcType = enumeration for whether it is RMC70 or RMC150, 2 = RMC70, 3 = RMC150
    'plotConfig =  the number of the plot to read. From RMCTools project pane > Plot Configurations
    'dataItems = number of data sets in the plot
    'plotSamples = number of plot samples
    'plotPeriod = the time period of the plot samples
    'regAddr =  an array of the register addresses for each dataItem.
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Private Function GetLoopTime() As Single

        Dim readData() As Single = New Single(1) {}
        Select Case rmcType.Text
            Case "RMC70"         'RMC75
                RMC.ReadFFile(20, 0, readData, 0, 1)
            Case "RMC150"     'RMC150
                RMC.ReadFFile(44, 0, readData, 0, 1)
            Case "RMC200"     'RMC200
                RMC.ReadFFile(18, 0, readData, 0, 1)
        End Select

        GetLoopTime = readData(0)
        Exit Function
    End Function

    Private Sub PlotCreateXMLFile(plotSet, regAddr, itemNames, units, plotPeriod, PlotId)
        Dim i, j As Integer
        Dim vbTab2
        Dim vbTab3
        Dim fileName As String
        Dim xmlData As String

        Dim thisDate As Date = Now

        Dim DateString As String = Str(thisDate.Year) + "_" + Str(thisDate.Month) + "_" + Str(thisDate.Day) + "_" + Str(thisDate.Hour) + Str(thisDate.Minute) + Str(thisDate.Second) + Str(thisDate.Millisecond)

        fileName = txtPlotPath.Text + "\" + txtPlotName.Text + "_" + DateString + ".rmcplots"
        'save tab sizes for indentation in plot file
        vbTab2 = vbTab + vbTab
        vbTab3 = vbTab + vbTab + vbTab

        Dim objWriter As New System.IO.StreamWriter(fileName)

        Dim penColors(16)
        penColors(0) = "16#E0E000"
        penColors(1) = "16#0000FF"
        penColors(2) = "16#FF00FF"
        penColors(3) = "16#FF0000"
        penColors(4) = "16#00E000"
        penColors(5) = "16#E0E000"
        penColors(6) = "16#00E0E0"
        penColors(7) = "16#C0C0C0"
        penColors(8) = "16#0066FF"
        penColors(9) = "16#CC99FF"
        penColors(10) = "16#CC99FF"
        penColors(11) = "16#663399"
        penColors(12) = "16#FF99CC"
        penColors(13) = "16#003399"
        penColors(14) = "16#00CC99"
        penColors(15) = "16#808000"

        'Create beginning of XML data
        xmlData = "<?xml version='1.0' encoding='utf-8'?>" + vbCrLf
        xmlData += "<PlotFolderRoot" + vbCrLf
        xmlData += vbTab + "Name=""Plot1""" + vbCrLf
        xmlData += vbTab + "Comment=""""" + vbCrLf
        xmlData += vbTab + "PlotFileVersion = ""1.1"">" + vbCrLf
        xmlData += vbTab + "<Plot" + vbCrLf
        xmlData += vbTab2 + "Name=""Controller1:NoName:1""" + vbCrLf
        xmlData += vbTab2 + "Comment = """"" + vbCrLf
        xmlData += vbTab2 + "SamplePeriodS=" + """" + CStr(plotPeriod) + """" + vbCrLf
        xmlData += vbTab2 + "CapturedDT=""9999-01-01 0 01:01:01.001""" + vbCrLf
        xmlData += vbTab2 + "LoopsPerInt=""1""" + vbCrLf
        xmlData += vbTab2 + "Items=""" + CStr(UBound(plotSet, 2) + 1) + """" + vbCrLf
        xmlData += vbTab2 + "FirstIntEver=""0"">" + vbCrLf
        xmlData += vbTab2 + "<Controller" + vbCrLf
        xmlData += vbTab3 + "ControllerType =" + """" + rmcType.Text + """" + "/>" + vbCrLf
        'Print #1, "</Plot>" + vbNewLine
        'Print #1, "</PlotFolderRoot>"

        'Create Item elements
        Dim itemLabels(0, 0) As String
        ReDim itemLabels(UBound(plotSet, 2), 3)

        For i = 0 To UBound(plotSet, 2)

            'add item
            itemLabels(i, 1) = "<Item"

            'add address
            itemLabels(i, 2) = "Address=""" & CStr(regAddr(i)) & """"

            'add type attribute
            itemLabels(i, 3) = "Type=""REAL""/>"

        Next 'Create Item elements

        'Add item elements to plot file
        For i = 0 To UBound(plotSet, 2)
            xmlData += vbTab2 + itemLabels(i, 1) + vbCrLf
            xmlData += vbTab3 + itemLabels(i, 2) + vbCrLf
            xmlData += vbTab3 + itemLabels(i, 3) + vbCrLf
        Next i

        'create plot labels
        Dim quantityLabels(0, 0) As String
        ReDim quantityLabels(UBound(plotSet, 1), 10)

        For i = 0 To UBound(plotSet, 2)

            'add Quantity
            quantityLabels(i, 0) = "<Quantity"

            'add Name attribute
            quantityLabels(i, 1) = "Name=" & """" & CStr(itemNames(i)) & """"

            'add Units attribute
            quantityLabels(i, 2) = "Units=" & """" & CStr(units(i)) & """"

            'add Formula attribute
            quantityLabels(i, 3) = "Formula=" & """" & CStr(regAddr(i)) & """"

            'add Filter attribute
            quantityLabels(i, 4) = "Filter=""False"""

            'add Type attribute
            quantityLabels(i, 5) = "Type=""REAL"""

            'add DecPlaces attribute
            quantityLabels(i, 6) = "DecPlaces=""3"""

            'add Graph attribute
            quantityLabels(i, 7) = "Graph=""True"""

            'add Pen attribute
            quantityLabels(i, 8) = "Pen=" & """" & CStr(penColors(i)) & """"

            'add AutoScale attribute
            quantityLabels(i, 9) = "AutoScale=""True"""

            'add AutoGroup attribute
            quantityLabels(i, 10) = "AutoGroup=""True""/>"

        Next i 'Create Sample elements

        'Add quanitity labels to plot file
        For i = 0 To UBound(plotSet, 2)
            xmlData += vbTab2 & quantityLabels(i, 0) + vbCrLf
            For j = 1 To 10
                xmlData += vbTab3 & quantityLabels(i, j) + vbCrLf
            Next j
        Next i

        objWriter.Write(xmlData)
        xmlData = vbNullString


        'Create plot data array
        Dim plotValues()
        ReDim plotValues(UBound(plotSet, 1))

        For i = 0 To UBound(plotSet, 1)

            'add Int attribute
            xmlData += "<Sample Int=""" + CStr(i) + """"

            'add Item attributes
            For j = 0 To UBound(plotSet, 2)

                xmlData += " Item" + CStr(j) + "=""" + CStr(plotSet(i, j)) + """"

            Next j

            xmlData += " />" + vbCrLf

            objWriter.Write(xmlData)
            xmlData = vbNullString

        Next i 'Create Sample elements

        xmlData += vbTab + "</Plot>"
        xmlData += "</PlotFolderRoot>"
        '********************************************



        objWriter.Write(xmlData)
        objWriter.Close()

    End Sub
    Private Sub PlotCreateCSVfile(plotSet, regAddr, itemNames, units, plotPeriod, PlotId)
        Dim i, j As Integer
        Dim fileName As String
        Dim csvData As String = ""

        Dim thisDate As Date = Now

        Dim DateString As String = Str(thisDate.Year) + "_" + Str(thisDate.Month) + "_" + Str(thisDate.Day) + "_" + Str(thisDate.Hour) + Str(thisDate.Minute) + Str(thisDate.Second) + Str(thisDate.Millisecond)

        fileName = txtPlotPath.Text + "\" + txtPlotName.Text + "_" + DateString + ".csv"

        Dim objWriter As New System.IO.StreamWriter(fileName)

        'Create plot data array
        Dim plotValues()
        ReDim plotValues(UBound(plotSet, 1))

        For i = 0 To UBound(plotSet, 2)

            'add Int attribute
            If i > 0 Then csvData += ","
            csvData += CStr(itemNames(i))
        Next

        csvData += vbCrLf

        objWriter.Write(csvData)
        csvData = vbNullString

        'add Item attributes
        For j = 0 To UBound(plotSet, 1)
            For i = 0 To UBound(plotSet, 2)
                If i > 0 Then csvData += ","
                csvData += CStr(plotSet(j, i))
            Next
            csvData += vbCrLf

            ' we write out each line to the file to prevent the vb string from getting too big,
            ' when the vb string gets big, performance gets terrible
            objWriter.Write(csvData)
            csvData = vbNullString
        Next j



        objWriter.Write(csvData)
        objWriter.Close()

    End Sub
End Class
